@@ -66,6 +66,17 @@ class BasePDFView(ReportPDFView):
         else:
             self.term = None
 
+    def guessPeriodGroup(self, meeting):
+        # XXX: this is a quick fix, evil in it's own way
+        if meeting.period is not None:
+            return meeting.period.title
+        return meeting.dtstart.time().isoformat()[:5]
+
+    def makeFileName(self, basename):
+        timestamp = datetime.now().strftime('%y%m%d%H%M')
+        return '%s_%s.pdf' % (basename, timestamp)
+
+
 
 class BaseStudentPDFView(BasePDFView):
     """A base class for all student PDF views"""
@@ -288,6 +299,10 @@ class StudentReportCardPDFView(BaseReportCardPDFView):
     def collectStudents(self):
         return [self.context]
 
+    @property
+    def filename(self):
+        return self.makeFileName('report_card_%s' % self.context.username)
+
 
 class GroupReportCardPDFView(BaseReportCardPDFView):
     """A view for printing a report card for each person in a group"""
@@ -295,6 +310,10 @@ class GroupReportCardPDFView(BaseReportCardPDFView):
     def collectStudents(self):
         return list(self.context.members)
 
+    @property
+    def filename(self):
+        return self.makeFileName('report_card_%s' % self.context.__name__)
+
 
 class BaseStudentDetailPDFView(BaseStudentPDFView):
     """The report card (PDF) base class"""
@@ -358,7 +377,7 @@ class BaseStudentDetailPDFView(BaseStudentPDFView):
             if jd is None:
                 continue
             for meeting in jd.recordedMeetings(student):
-                period = meeting.period_id[:5]
+                period = self.guessPeriodGroup(meeting)
                 day = meeting.dtstart
                 grade = jd.getGrade(student, meeting)
                 result = ''
@@ -375,7 +394,13 @@ class BaseStudentDetailPDFView(BaseStudentPDFView):
                 periods[period] = 0
         periods = list(periods.keys())
 
-        widths = '4cm' + ',1cm' * len(periods)
+        widths = '4cm'
+        for period in periods:
+            width = 1
+            if len(period) > 4:
+                width = float(len(period)) / 4.5
+            widths += ',%fcm' % width
+
         rows = []
         for day in sorted(data):
             scores = [''] * len(periods)
@@ -414,6 +439,10 @@ class StudentDetailPDFView(BaseStudentDetailPDFView):
     def collectStudents(self):
         return [self.context]
 
+    @property
+    def filename(self):
+        return self.makeFileName('student_detail_%s' % self.context.username)
+
 
 class GroupDetailPDFView(BaseStudentDetailPDFView):
     """A view for printing a report card for each person in a group"""
@@ -421,6 +450,10 @@ class GroupDetailPDFView(BaseStudentDetailPDFView):
     def collectStudents(self):
         return list(self.context.members)
 
+    @property
+    def filename(self):
+        return self.makeFileName('student_detail_%s' % self.context.__name__)
+
 
 class FailingReportPDFView(BasePDFView):
     """A view for printing a report of all the students failing an activity"""
@@ -445,6 +478,10 @@ class FailingReportPDFView(BasePDFView):
         return _('Failures by Term Report: ${term}',
                  mapping={'term': self.term.title})
 
+    @property
+    def filename(self):
+        return self.makeFileName('fail_%s' % self.term.__name__)
+
     def worksheet_heading(self):
         return _('Report Sheet:')
 
@@ -555,6 +592,10 @@ class AbsencesByDayPDFView(BasePDFView):
     def title(self):
         return _('Absences By Day Report')
 
+    @property
+    def filename(self):
+        return self.makeFileName('abs_day_%s' % self.context.__name__)
+
     def getDay(self):
         day = self.request.get('day', None)
         if day is None:
@@ -588,7 +629,7 @@ class AbsencesByDayPDFView(BasePDFView):
                 for meeting in jd.recordedMeetings(student):
                     if not self.compareDates(meeting.dtstart, day):
                         continue
-                    period = meeting.period_id[:5]
+                    period = self.guessPeriodGroup(meeting)
                     grade = jd.getGrade(student, meeting)
                     result = ''
                     if grade == 'n':
@@ -658,6 +699,11 @@ class SectionAbsencesPDFView(BasePDFView):
     def title(self):
         return _('Absences by Section Report')
 
+    @property
+    def filename(self):
+        courses = [c.__name__ for c in self.context.courses]
+        return self.makeFileName('abs_section_%s' % '_'.join(courses))
+
     def course_heading(self):
         return _('Course')
 
@@ -735,6 +781,7 @@ class GradebookPDFView(BasePDFView, GradebookOverview):
     def pages(self):
         results = []
         activities = list(self.activities())
+
         table = self.table()
 
         num_rows = len(table)
@@ -764,14 +811,14 @@ class GradebookPDFView(BasePDFView, GradebookOverview):
             else:
                 end_col = next_col
 
-            rows = deepcopy(table)[start_row:end_row]
+            rows = [dict(r) for r in table[start_row:end_row]]
             for row in rows:
                 row['grades'] = row['grades'][start_col:end_col]
 
             page = {
                 'widths': self.widths(start_col, end_col),
                 'rows': rows,
-                'cols': deepcopy(activities)[start_col:end_col],
+                'cols': activities[start_col:end_col],
                 }
             results.append(page)
 
