@@ -74,6 +74,10 @@ class IActivities(interfaces.IRequirement):
     '''A list of worksheets containing activities that must be fulfilled in a
        course or section.'''
 
+    worksheets = Attribute("""all non-hidden worksheets""")
+
+    all_worksheets = Attribute("""all worksheets hidden or not""")
+
     def resetCurrentWorksheet():
         """Reset the currently active worksheet to first or None."""
 
@@ -89,6 +93,18 @@ class IActivities(interfaces.IRequirement):
     contains('.IWorksheet')
 
 
+class ICourseActivities(interfaces.IRequirement):
+    """Container of Course Worksheet Templates that can be deployed"""
+
+    contains('.ICourseWorksheet')
+
+
+class ICourseDeployedWorksheets(interfaces.IRequirement):
+    """Container of Deployed Course Worksheets (by term)"""
+
+    contains('.IWorksheet')
+
+
 class IWorksheet(interfaces.IRequirement):
     '''A list of activities that must be fulfilled in a course or section.'''
 
@@ -111,12 +127,18 @@ class IWorksheet(interfaces.IRequirement):
         """Set the weight for the given category.  Any numeric type is
            acceptable"""
 
+    def isCourseWorksheet():
+        """return True if worksheet was deployed as course worksheet"""
+
+    def canAverage():
+        """return True if activities have scoresystems that can be averaged"""
+
     containers(IActivities)
     contains('.IActivity')
 
 
 class IReportWorksheet(interfaces.IRequirement):
-    '''A list of report card activities that get copied into sections.'''
+    '''A worksheet template to get copied into section gradebooks.'''
 
     containers(IGradebookTemplates, IGradebookDeployed)
     contains('.IReportActivity')
@@ -126,13 +148,23 @@ class IReportWorksheet(interfaces.IRequirement):
         description=_(u'Identifies the report sheet in teacher gradebooks.'))
 
 
+class ICourseWorksheet(interfaces.IRequirement):
+    '''A worksheet template to get copied into section gradebooks.'''
+
+    contains('.IActivity')
+
+    title = zope.schema.TextLine(
+        title=_(u'Title'),
+        description=_(u'Identifies the course worksheet in gradebooks.'))
+
+
 class IActivity(interfaces.IRequirement):
     '''An activity to be graded'''
 
     due_date = zope.schema.Date(
         title=_("Due Date"),
         description=_("The date the activity is due to be graded."),
-        required=True)
+        required=False)
 
     label = zope.schema.TextLine(
         title=_(u"Label"),
@@ -141,7 +173,6 @@ class IActivity(interfaces.IRequirement):
 
     description = zope.schema.Text(
         title=_("Description"),
-        description=_("A detailed description of the activity."),
         required=False)
 
     category = zope.schema.Choice(
@@ -164,7 +195,7 @@ class IActivity(interfaces.IRequirement):
 
 
 class IReportActivity(IActivity):
-    '''An activity to be deployed to section activities'''
+    '''A report card activity to be deployed to section activities'''
 
     containers(IReportWorksheet)
 
@@ -406,7 +437,6 @@ class IExternalActivity(zope.interface.Interface):
 
     description = zope.schema.Text(
         title=_("Description"),
-        description=_("A detailed description of the external activity."),
         required=False)
 
     def getGrade(student):
